#!/usr/bin/perl

$debug = 0;
#$debug = 1;

use Time::Local;

$now = time();

($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime();

$timestamp = sprintf( "%04d%02d%02d%02d%02d%02d", $year + 1900, $mon + 1, $mday, $hour, $min, $sec);

$SUDO = "/usr/bin/sudo";
$LVDISPLAY = "$SUDO lvdisplay -m 2>/dev/null";

$PARTX = "$SUDO partx";

$uname = `uname -r`;
$rhel4 = 0;

if ( $uname =~ /el[56]/ ) {
  $IOSTAT = "/usr/bin/iostat -xkd 2 2";
} else {
  $rhel4 = 1;
  $IOSTAT = "/usr/bin/iostat -xd 2 2";
}

print "DEBUG:rhel4=$rhel4\n" if $debug;

$DF = "/bin/df -P";
$SWAPON = "/sbin/swapon -s";

$HOSTNAME = $ENV{HOSTNAME};

$HOSTNAME =~ s/$/.stanwell.com/ if ( ! ($HOSTNAME =~ /\.stanwell\.com/) );

# Build list of devices for File Systems.

open( DF, "$DF |" );

while ( <DF> ) {

  chomp;

  s/\t/ /g;
  s/  */ /g;

  if ( ! (/^Filesystem / or /^none/) ) {
    ($DF_FileSystem, $NULL, $NULL, $NULL, $NULL, $DF_Mount) = split/ /;
    $Mount_Array{$DF_FileSystem} = $DF_Mount;
    print "DBG:FileSystems:$DF_FileSystem:$DF_Mount\n" if ( $debug );
  }
  
}

close( DF );

# Build list of devices for Swap.

open( SWAPON, "$SWAPON |" );

while ( <SWAPON> ) {

  chomp;

  s/\t/ /g;
  s/  */ /g;

  if ( ! (/^Filename / or /^none/) ) {
    ($DF_FileSystem) = split/ /;
    $Mount_Array{$DF_FileSystem} = 'swap';
    print "DBG:Swap:$DF_FileSystem:swap\n" if ( $debug );
  }
  
}
close( SWAPON );

# Get Stats for each device.

open( IOSTAT, "$IOSTAT |" );

$count = 0;

while ( <IOSTAT> ) {

  print if ( $debug );
  chomp;

  if ( $count == 2 ) {

    s/\t/ /g;
    s/  */ /g;
    s/^ //;

    ($disk) = split/ /;

    $disk = "/dev/".$disk;

    if ( $rhel4 ) {
      ($NULL, $NULL, $NULL,
       $read_sec{$disk}, $write_sec{$disk}, 
       $NULL, $NULL,
       $kread_sec{$disk}, $kwrite_sec{$disk},
       $NULL,
       $avgqusz{$disk}, $await{$disk}, $svctm{$disk}, $util{$disk} ) = split/ /;
    } else {
      ($NULL, $NULL, $NULL,
       $read_sec{$disk}, $write_sec{$disk}, 
       $kread_sec{$disk}, $kwrite_sec{$disk},
       $NULL,
       $avgqusz{$disk}, $await{$disk}, $svctm{$disk}, $util{$disk} ) = split/ /;
    }

    $IOStatLine = $_;

    open( PARTX, "$PARTX $disk 2>/dev/null |" );

    while ( <PARTX> ) {

      s/^#.//;

      ($DiskNo, $NULL) = split/:/;

      $NewDisk = "$disk$DiskNo";

      if ( $rhel4 ) {
        ($NULL, $NULL, $NULL,
         $read_sec{$NewDisk}, $write_sec{$NewDisk}, 
         $NULL, $NULL,
         $kread_sec{$NewDisk}, $kwrite_sec{$NewDisk},
         $NULL,
         $avgqusz{$NewDisk}, $await{$NewDisk}, $svctm{$NewDisk}, $util{$NewDisk} ) = split/ /,$IOStatLine;
      } else {
        ($NULL, $NULL, $NULL,
         $read_sec{$NewDisk}, $write_sec{$NewDisk}, 
         $kread_sec{$NewDisk}, $kwrite_sec{$NewDisk},
         $NULL,
         $avgqusz{$NewDisk}, $await{$NewDisk}, $svctm{$NewDisk}, $util{$NewDisk} ) = split/ /,$IOStatLine;
      }

    }
  }

  $count++ if ( /^Device:/ );

}

close( IOSTAT );

$old_disk = "";

# Get Logical Volumes that are used.

my $FoundLV = FALSE;

open( LV, "$LVDISPLAY |" );

while ( <LV> ) {

  chomp;

  s/\t/ /g;
  s/  */ /g;
  s/^ //;

#  print "DEBUG:LV:$_\n" if ( $debug );
  if ( /LV Name/ ) {

    if ( $FoundLV ) {
      delete  $Mount_Array{$FoundDevice};
      $FoundLV = FALSE;
    }

    ($NULL, $NULL, $LV_Name) = split/ /;
    $LV_Name =~ s/.*\///;

  }

  if ( /VG Name/ ) {
    ($NULL, $NULL, $VG_Name) = split/ /;
    $DF_FileSystem = "/dev/mapper/$VG_Name"."-".$LV_Name;

    print "DEBUG:LVName:$LV_Name:VGName:$VG_Name:LVPart:$LV_Name\n" if $debug;
  }

  if ( /Block device/ ) {
    ($NULL, $DevMap) = split/:/;
    $DevMap = "dm-$DevMap";
    print "DEBUG:DM=$DevMap\n" if $debug;
  }

  if ( /Physical volume/ ) {

    @VolGroups = ( "/dev/mapper/$VG_Name-$LV_Name", "/dev/$DevMap");

    foreach $Device (@VolGroups) {
      if ( $Mount_Array{$Device} ne '' ) {
        $FoundLV = TRUE;

        ($NULL, $NULL, $disk) = split/ /;

        $DATA_STREAM = sprintf( "%s%s;fsio;%s;%s;%s:%s:%s:%s:%s:%s:%s:%s:%s:%s;\n",
                                $DATA_STREAM, $timestamp, $Mount_Array{$Device},
                                $disk, $read_sec{$disk}, $write_sec{$disk}, 
                                $kread_sec{$disk}, $kwrite_sec{$disk}, $avgqusz{$disk},
                                "0.0", $await{$disk}, $svctm{$disk}, "0.0", $util{$disk} );
        $FoundDevice = $Device;
      }
    }
  }
}

if ( $FoundLV ) {
  delete  $Mount_Array{$FoundDevice};
}
close( LV );

# Get device stat for remainder of file systems.

foreach my $disk ( sort keys %Mount_Array ){
  if ( $read_sec{$disk} ne '' ) {
    $DATA_STREAM = sprintf( "%s%s;fsio;%s;%s;%s:%s:%s:%s:%s:%s:%s:%s:%s:%s;\n",
                            $DATA_STREAM, $timestamp, $Mount_Array{$disk},
                            $disk, $read_sec{$disk}, $write_sec{$disk}, 
                            $kread_sec{$disk}, $kwrite_sec{$disk}, $avgqusz{$disk},
                            "0.0", $await{$disk}, $svctm{$disk}, "0.0", $util{$disk} );
  }
}

`$ENV{BB} xymon.stanwell.com "data $HOSTNAME.fsio \n$DATA_STREAM"`;
print "$ENV{BB} xymon.stanwell.com data $HOSTNAME.fsio\n $DATA_STREAM\n" if ( $debug );
